@@ -16,8 +16,11 @@
 
 import fileinput
 import os
+import platform
 import re
+import shutil
 import subprocess
+import sys
 import time
 import uuid
 import xdg
@@ -25,12 +28,21 @@ from distutils import dir_util
 
 import fixtures
 import pexpect
-from unittest import mock
+import requests
 import testtools
+import yaml
+from unittest import mock
 from testtools import content
 from testtools.matchers import MatchesRegex
+from snapcraft import ProjectOptions as _ProjectOptions
+from snapcraft.tests import (
+    fixture_setup,
+    subprocess_utils
+)
 
-from snapcraft.tests import fixture_setup
+
+class RegisterError(Exception):
+    pass
 
 
 class TestCase(testtools.TestCase):
@@ -94,9 +106,12 @@ class TestCase(testtools.TestCase):
         self.stage_dir = 'stage'
         self.prime_dir = 'prime'
 
+        self.deb_arch = _ProjectOptions().deb_arch
+        self.distro_series = platform.linux_distribution()[2]
+
     def run_snapcraft(
             self, command, project_dir=None, debug=True,
-            pre_func=lambda: None):
+            pre_func=lambda: None, env=None):
         if project_dir:
             self.copy_project_to_cwd(project_dir)
 
@@ -109,7 +124,8 @@ class TestCase(testtools.TestCase):
             pre_func()
             snapcraft_output = subprocess.check_output(
                 snapcraft_command + command,
-                stderr=subprocess.STDOUT, universal_newlines=True)
+                stderr=subprocess.STDOUT, universal_newlines=True,
+                env=env)
         except subprocess.CalledProcessError as e:
             self.addDetail('output', content.text_content(e.output))
             raise
@@ -170,6 +186,178 @@ class TestCase(testtools.TestCase):
             output = exception.output
         return output
 
+    def set_stage_package_version(
+            self, snapcraft_yaml_path, part, package, version=None):
+        return self.set_package_version(
+            'stage-packages', snapcraft_yaml_path, part, package, version)
+
+    def set_build_package_version(
+            self, snapcraft_yaml_path, part, package, version=None):
+        return self.set_package_version(
+            'build-packages', snapcraft_yaml_path, part, package, version)
+
+    def set_package_version(
+            self, type_, snapcraft_yaml_path, part, package, version=None):
+        # This doesn't handle complex package syntax.
+        with open(snapcraft_yaml_path) as snapcraft_yaml_file:
+            snapcraft_yaml = yaml.load(snapcraft_yaml_file)
+        if part:
+            packages = snapcraft_yaml['parts'][part].get(type_, [])
+        else:
+            packages = snapcraft_yaml.get(type_, [])
+        for index, package_in_yaml in enumerate(packages):
+            if package_in_yaml.split('=')[0] == package:
+                if version is None:
+                    version = get_package_version(
+                        package, self.distro_series, self.deb_arch)
+
+                packages[index] = '{}={}'.format(package, version)
+                break
+        else:
+            self.fail("The part {} doesn't have a package {}".format(
+                part, package))
+
+        with open(snapcraft_yaml_path, 'w') as snapcraft_yaml_file:
+            yaml.dump(snapcraft_yaml, snapcraft_yaml_file)
+        return version
+
+    def set_build_package_architecture(
+            self, snapcraft_yaml_path, part, package, architecture):
+        # This doesn't handle complex package syntax.
+        with open(snapcraft_yaml_path) as snapcraft_yaml_file:
+            snapcraft_yaml = yaml.load(snapcraft_yaml_file)
+        packages = snapcraft_yaml['parts'][part]['build-packages']
+        for index, package_in_yaml in enumerate(packages):
+            if package_in_yaml == package:
+                packages[index] = '{}:{}'.format(package, architecture)
+                break
+        else:
+            self.fail("The part {} doesn't have a package {}".format(
+                part, package))
+
+        with open(snapcraft_yaml_path, 'w') as snapcraft_yaml_file:
+            yaml.dump(snapcraft_yaml, snapcraft_yaml_file)
+
+
+class BzrSourceBaseTestCase(TestCase):
+
+    def setUp(self):
+        super().setUp()
+        if shutil.which('bzr') is None:
+            self.skipTest('bzr is not installed')
+
+    def init_source_control(self):
+        subprocess.check_call(
+            ['bzr', 'init', '.'],
+            stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
+        subprocess.check_call(
+            ['bzr', 'whoami', '--branch', '"Example Dev <dev@example.com>"'])
+
+    def commit(self, message, unchanged=False):
+        command = ['bzr', 'commit', '-m', message]
+        if unchanged:
+            command.append('--unchanged')
+        subprocess.check_call(
+            command, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
+
+    def get_revno(self, path=None):
+        command = ['bzr', 'revno', '-r', '-1']
+        if path:
+            command.append(path)
+        return subprocess.check_output(
+            command, universal_newlines=True).strip()
+
+
+class GitSourceBaseTestCase(TestCase):
+
+    def setUp(self):
+        super().setUp()
+        if shutil.which('git') is None:
+            self.skipTest('git is not installed')
+
+    def init_source_control(self):
+        subprocess.check_call(
+            ['git', 'init', '.'], stdout=subprocess.DEVNULL)
+        subprocess.check_call(
+            ['git', 'config', '--local', 'user.name', '"Example Dev"'])
+        subprocess.check_call(
+            ['git', 'config', '--local', 'user.email', 'dev@example.com'])
+
+    def add_file(self, file_path):
+        subprocess.check_call(
+            ['git', 'add', file_path], stdout=subprocess.DEVNULL)
+
+    def commit(self, message, allow_empty=False):
+        command = ['git', 'commit', '-m', message]
+        if allow_empty:
+            command.append('--allow-empty')
+        subprocess.check_call(command, stdout=subprocess.DEVNULL)
+
+    def tag(self, tag_name):
+        subprocess.check_call(
+            ['git', 'tag', '-a', '-m', tag_name, tag_name],
+            stdout=subprocess.DEVNULL)
+
+    def get_revno(self):
+        return subprocess_utils.call_with_output(
+            ['git', 'rev-list', 'HEAD', '--max-count=1'])
+
+
+class HgSourceBaseTestCase(TestCase):
+
+    def setUp(self):
+        super().setUp()
+        if shutil.which('hg') is None:
+            self.skipTest('mercurial is not installed')
+
+    def init_source_control(self):
+        subprocess.check_call(['hg', 'init', '.'])
+
+    def commit(self, message, file_):
+        subprocess.check_call(
+            ['hg', 'commit', '-m', message, '--user',
+             '"Example Dev"', '-A', file_])
+
+    def get_revno(self, path=None):
+        command = ['hg', 'log', '--template', '"{desc}"', '-r', '-1']
+        if path:
+            command.extend(['--cwd', path])
+        return subprocess.check_output(
+            command, universal_newlines=True).strip()
+
+    def get_id(self):
+        return subprocess_utils.call_with_output(['hg', 'id']).split()[0]
+
+
+class SubversionSourceBaseTestCase(TestCase):
+
+    def setUp(self):
+        super().setUp()
+        if shutil.which('svn') is None:
+            self.skipTest('svn is not installed')
+
+    def init_source_control(self):
+        subprocess.check_call(
+            ['svnadmin', 'create', 'repo'], stdout=subprocess.DEVNULL)
+
+    def checkout(self, source, destination):
+        subprocess.check_call(
+            ['svn', 'checkout', source, destination],
+            stdout=subprocess.DEVNULL)
+
+    def add(self, file_path, cwd=None):
+        subprocess.check_call(
+            ['svn', 'add', file_path], stdout=subprocess.DEVNULL, cwd=cwd)
+
+    def commit(self, message, cwd=None):
+        subprocess.check_call(
+            ['svn', 'commit', '-m', message],
+            stdout=subprocess.DEVNULL, cwd=cwd)
+
+    def update(self, cwd=None):
+        subprocess.check_call(
+            ['svn', 'update'], stdout=subprocess.DEVNULL, cwd=cwd)
+
 
 class StoreTestCase(TestCase):
 
@@ -198,7 +386,7 @@ class StoreTestCase(TestCase):
 
     def logout(self):
         output = self.run_snapcraft('logout')
-        expected = (r'.*Clearing credentials for Ubuntu One SSO.\n.*\n'
+        expected = (r'.*Clearing credentials for Ubuntu One SSO.\n'
                     r'Credentials cleared.\n.*')
         self.assertThat(output, MatchesRegex(expected, flags=re.DOTALL))
 
@@ -206,11 +394,26 @@ class StoreTestCase(TestCase):
         command = ['register', snap_name]
         if private:
             command.append('--private')
-        self.run_snapcraft(command)
-        # sleep a few seconds to avoid hitting the store restriction on
-        # following registrations.
-        if wait:
-            time.sleep(self.test_store.register_delay)
+        process = pexpect.spawn(self.snapcraft_command, command)
+        process.expect(r'.*\[y/N\]: ')
+        process.sendline('y')
+        try:
+            process.expect_exact(
+                'Congrats! You are now the publisher of {!r}.'.format(
+                    snap_name))
+        except pexpect.exceptions.EOF:
+            wait_error_regex = (
+                '.*You must wait (\d+) seconds before trying to register your '
+                'next snap.*')
+            output = process.before.decode(sys.getfilesystemencoding())
+            match = re.search(wait_error_regex, output)
+            if wait and match:
+                time.sleep(int(match.group(1)))
+                # This could get stuck for ever if the user is registering
+                # other snaps in parallel.
+                self.register(snap_name, private, wait)
+            else:
+                raise RegisterError(output)
 
     def register_key(self, key_name, email=None, password=None,
                      expect_success=True):
@@ -264,17 +467,34 @@ class StoreTestCase(TestCase):
         process.close()
         return process.exitstatus
 
+    def get_unique_name(self, prefix='snapcrafttest'):
+        """Return a unique snap name.
+
+        It uses a UUIDv4 to create unique names and limits its full size
+        to 40 chars (as defined in the snap specification).
+        """
+        unique_id = uuid.uuid4().int
+        return '{}-{}'.format(prefix, unique_id)[:40]
+
+    def get_unique_version(self):
+        """Return a unique snap version.
+
+        It uses a UUIDv4 to create unique version and limits its full size
+        to 32 chars (as defined in the snap specification).
+        """
+        unique_id = uuid.uuid4().int
+        return '{}'.format(unique_id)[:32]
+
     def update_name_arch_and_version(self, name=None, arch=None,
                                      version=None):
-        unique_id = uuid.uuid4().int
         if name is None:
-            name = 'u1test-{}'.format(unique_id)
+            name = self.get_unique_name()
         if version is None:
-            # The maximum size is 32 chars.
-            version = str(unique_id)[:32]
+            version = self.get_unique_version()
         if arch is None:
             arch = 'amd64'
-        for line in fileinput.input('snapcraft.yaml', inplace=True):
+        for line in fileinput.input(
+                os.path.join('snap', 'snapcraft.yaml'), inplace=True):
             if 'name: ' in line:
                 print('name: {}'.format(name))
             elif 'version: ' in line:
@@ -285,13 +505,12 @@ class StoreTestCase(TestCase):
                 print(line)
 
     def update_name_and_version(self, name=None, version=None):
-        unique_id = uuid.uuid4().int
         if name is None:
-            name = 'u1test-{}'.format(unique_id)
+            name = self.get_unique_name()
         if version is None:
-            # The maximum size is 32 chars.
-            version = str(unique_id)[:32]
-        for line in fileinput.input('snapcraft.yaml', inplace=True):
+            version = self.get_unique_version()
+        for line in fileinput.input(
+                os.path.join('snap', 'snapcraft.yaml'), inplace=True):
             if 'name: ' in line:
                 print('name: {}'.format(name))
             elif 'version: ' in line:
@@ -318,7 +537,8 @@ class StoreTestCase(TestCase):
             process.expect(expected_error)
         else:
             for v in validations:
-                process.expect('Signing validation {}'.format(v))
+                process.expect(
+                    'Signing validations assertion for {}'.format(v))
         process.expect(pexpect.EOF)
         process.close()
         return process.exitstatus
@@ -333,11 +553,11 @@ class StoreTestCase(TestCase):
         if expect_success:
             if local:
                 process.expect(
-                    'Build assertion {}-build saved to disk.'.format(
+                    'Build assertion .*{}-build saved to disk.'.format(
                         snap_filename))
             else:
                 process.expect(
-                    'Build assertion {}-build pushed.'.format(snap_filename))
+                    'Build assertion .*{}-build pushed.'.format(snap_filename))
 
         process.expect(pexpect.EOF)
         process.close()
@@ -364,3 +584,19 @@ class StoreTestCase(TestCase):
         process.expect(pexpect.EOF)
         process.close()
         return process.exitstatus
+
+
+def get_package_version(package_name, series, deb_arch):
+    # http://people.canonical.com/~ubuntu-archive/madison.cgi?package=hello&a=amd64&c=&s=zesty&text=on
+    params = {
+        'package': package_name,
+        's': '{0},{0}-updates,{0}-security'.format(series),
+        'a': deb_arch,
+        'text': 'on',
+    }
+    query = requests.get('http://people.canonical.com/~ubuntu-archive/'
+                         'madison.cgi', params)
+    query.raise_for_status()
+    package = query.text.strip().split('\n')[-1]
+    package_status = [i.strip() for i in package.strip().split('|')]
+    return package_status[1]
