@@ -1,6 +1,6 @@
 # -*- Mode:Python; indent-tabs-mode:nil; tab-width:4 -*-
 #
-# Copyright (C) 2016 Canonical Ltd
+# Copyright (C) 2016-2017 Canonical Ltd
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 3 as
@@ -18,6 +18,7 @@ import os
 
 from testtools.matchers import (
     DirExists,
+    Equals,
     FileExists,
     Not
 )
@@ -30,11 +31,10 @@ class CleanPullStepPulledTestCase(integration_tests.TestCase):
     def setUp(self):
         super().setUp()
 
-        self.project_dir = 'independent-parts'
-        self.run_snapcraft('pull', self.project_dir)
-        self.partsdir = os.path.join(self.project_dir, 'parts')
-        self.part1_sourcedir = os.path.join(self.partsdir, 'part1', 'src')
-        self.part2_sourcedir = os.path.join(self.partsdir, 'part2', 'src')
+        self.copy_project_to_cwd('independent-parts')
+        self.run_snapcraft('pull')
+        self.part1_sourcedir = os.path.join(self.parts_dir, 'part1', 'src')
+        self.part2_sourcedir = os.path.join(self.parts_dir, 'part2', 'src')
 
     def assert_files_exist(self):
         self.assertThat(os.path.join(self.part1_sourcedir, 'file1'),
@@ -45,25 +45,24 @@ class CleanPullStepPulledTestCase(integration_tests.TestCase):
     def test_clean_pull_step(self):
         self.assert_files_exist()
 
-        self.run_snapcraft(['clean', '--step=pull'], self.project_dir)
+        self.run_snapcraft(['clean', '--step=pull'])
         self.assertThat(self.part1_sourcedir, Not(DirExists()))
         self.assertThat(self.part2_sourcedir, Not(DirExists()))
 
         # Now try to pull again
-        self.run_snapcraft('pull', self.project_dir)
+        self.run_snapcraft('pull')
         self.assert_files_exist()
 
     def test_clean_pull_step_single_part(self):
         self.assert_files_exist()
 
-        self.run_snapcraft(['clean', 'part1', '--step=pull'],
-                           self.project_dir)
+        self.run_snapcraft(['clean', 'part1', '--step=pull'])
         self.assertThat(self.part1_sourcedir, Not(DirExists()))
         self.assertThat(os.path.join(self.part2_sourcedir, 'file2'),
                         FileExists())
 
         # Now try to pull again
-        self.run_snapcraft('pull', self.project_dir)
+        self.run_snapcraft('pull')
         self.assert_files_exist()
 
 
@@ -72,17 +71,14 @@ class CleanPullStepPrimedTestCase(integration_tests.TestCase):
     def setUp(self):
         super().setUp()
 
-        self.project_dir = 'independent-parts'
-        self.run_snapcraft('prime', self.project_dir)
+        self.copy_project_to_cwd('independent-parts')
+        self.run_snapcraft('prime')
 
-        self.snapdir = os.path.join(self.project_dir, 'prime')
-        self.snap_bindir = os.path.join(self.snapdir, 'bin')
-        self.stagedir = os.path.join(self.project_dir, 'stage')
-        self.stage_bindir = os.path.join(self.stagedir, 'bin')
-        self.partsdir = os.path.join(self.project_dir, 'parts')
+        self.snap_bindir = os.path.join(self.prime_dir, 'bin')
+        self.stage_bindir = os.path.join(self.stage_dir, 'bin')
         self.parts = {}
         for part in ['part1', 'part2']:
-            partdir = os.path.join(self.partsdir, part)
+            partdir = os.path.join(self.parts_dir, part)
             self.parts[part] = {
                 'partdir': partdir,
                 'sourcedir': os.path.join(partdir, 'src'),
@@ -106,24 +102,32 @@ class CleanPullStepPrimedTestCase(integration_tests.TestCase):
     def test_clean_pull_step(self):
         self.assert_files_exist()
 
-        self.run_snapcraft(['clean', '--step=pull'], self.project_dir)
-        self.assertThat(self.stagedir, Not(DirExists()))
-        self.assertThat(self.snapdir, Not(DirExists()))
+        output = self.run_snapcraft(['clean', '--step=pull'], debug=False)
+        self.assertThat(self.stage_dir, Not(DirExists()))
+        self.assertThat(self.prime_dir, Not(DirExists()))
 
         for part_name, part in self.parts.items():
             self.assertThat(part['builddir'], Not(DirExists()))
             self.assertThat(part['installdir'], Not(DirExists()))
             self.assertThat(part['sourcedir'], Not(DirExists()))
 
+        # Assert that the priming and staging areas were removed wholesale, not
+        # a part at a time (since we didn't specify any parts).
+        output = output.strip().split('\n')
+        self.expectThat(output, Equals([
+            'Cleaning up priming area',
+            'Cleaning up staging area',
+            'Cleaning up parts directory'
+        ]))
+
         # Now try to prime again
-        self.run_snapcraft('prime', self.project_dir)
+        self.run_snapcraft('prime')
         self.assert_files_exist()
 
     def test_clean_pull_step_single_part(self):
         self.assert_files_exist()
 
-        self.run_snapcraft(['clean', 'part1', '--step=pull'],
-                           self.project_dir)
+        self.run_snapcraft(['clean', 'part1', '--step=pull'])
         self.assertThat(os.path.join(self.stage_bindir, 'file1'),
                         Not(FileExists()))
         self.assertThat(os.path.join(self.stage_bindir, 'file2'), FileExists())
@@ -146,5 +150,5 @@ class CleanPullStepPrimedTestCase(integration_tests.TestCase):
             FileExists())
 
         # Now try to prime again
-        self.run_snapcraft('prime', self.project_dir)
+        self.run_snapcraft('prime')
         self.assert_files_exist()
