@@ -14,6 +14,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+import os
 import subprocess
 
 import integration_tests
@@ -21,12 +22,18 @@ import integration_tests
 
 class StagePackageVersionTestCase(integration_tests.TestCase):
 
-    def test_stage_package_gets_version(self):
+    def test_stage_package_with_invalid_version_must_fail(self):
+        self.copy_project_to_cwd('stage-packages-missing-dependency')
+        self.set_stage_package_version(
+            os.path.join('snap', 'snapcraft.yaml'),
+            part='part-with-stage-packages',
+            package='haskell-doc', version='invalid')
         error = self.assertRaises(
             subprocess.CalledProcessError,
-            self.run_snapcraft, 'pull', 'stage-package-version')
+            self.run_snapcraft, 'pull')
         self.assertIn(
-            "Error downloading stage packages for part 'hello': "
-            "The package 'hello=x.y-z' was not found.",
+            "Error downloading stage packages for part "
+            "'part-with-stage-packages': "
+            "The package 'haskell-doc=invalid' was not found.",
             str(error.output)
         )
