@@ -1,6 +1,6 @@
 # -*- Mode:Python; indent-tabs-mode:nil; tab-width:4 -*-
 #
-# Copyright (C) 2016 Canonical Ltd
+# Copyright (C) 2016-2017 Canonical Ltd
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 3 as
@@ -27,16 +27,16 @@ import integration_tests
 class YamlInSubdirTestCase(integration_tests.TestCase):
 
     def test_stage(self):
-        project_dir = 'yaml-in-subdir'
-        subdir = os.path.join(project_dir, 'subdir')
-        self.run_snapcraft('stage', project_dir, subdir)
+        self.copy_project_to_cwd('yaml-in-subdir')
+        os.chdir('subdir')
+        self.run_snapcraft('stage')
 
-        expected_file = os.path.join(subdir, 'stage', 'file')
+        expected_file = os.path.join(self.stage_dir, 'file')
         self.assertThat(expected_file, FileExists())
         with open(expected_file) as f:
             self.assertThat(f.read(), Equals("I'm a file\n"))
 
-        expected_file = os.path.join(subdir, 'stage', 'subdirfile')
+        expected_file = os.path.join(self.stage_dir, 'subdirfile')
         self.assertThat(expected_file, FileExists())
         with open(expected_file) as f:
             self.assertThat(f.read(), Equals("I'm in the subdir\n"))
