@@ -15,6 +15,7 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import configparser
+import logging
 import os
 import urllib.parse
 
@@ -22,6 +23,10 @@ from xdg import BaseDirectory
 
 from snapcraft.storeapi import constants
 
+LOCAL_CONFIG_FILENAME = '.snapcraft/snapcraft.cfg'
+
+logger = logging.getLogger(__name__)
+
 
 class Config(object):
     """Hold configuration options in sections.
@@ -66,6 +71,17 @@ class Config(object):
         return True
 
     def load(self):
+        # Local configurations (per project) are supposed to be static.
+        # That's why it's only checked for 'loading' and never written to.
+        # Essentially, all authentication-related changes, like login/logout
+        # or macaroon-refresh, will not be persisted for the next runs.
+        if os.path.exists(LOCAL_CONFIG_FILENAME):
+            self.parser.read(LOCAL_CONFIG_FILENAME)
+            logger.warn(
+                'Using local configuration (`{}`), changes will '
+                'not be persisted.'.format(LOCAL_CONFIG_FILENAME))
+            return
+
         self.filename = BaseDirectory.load_first_config(
             'snapcraft', 'snapcraft.cfg')
         if self.filename and os.path.exists(self.filename):
