@@ -14,5 +14,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+from snapcraft.internal import cache             # noqa
+from snapcraft.internal import deltas            # noqa
 from snapcraft.internal import states            # noqa
 from snapcraft.internal.project_loader import load_config  # noqa
