@@ -25,8 +25,8 @@ from xdg import BaseDirectory
 
 from snapcraft.internal.indicators import download_requests_stream
 from snapcraft.internal.common import get_terminal_width
-from snapcraft.internal.errors import SnapcraftPartMissingError
 from snapcraft.internal import (
+    errors,
     deprecations,
     pluginhandler,
     project_loader,
@@ -102,7 +102,7 @@ class _RemoteParts(_Base):
         try:
             remote_part = self._parts[part_name].copy()
         except KeyError:
-            raise SnapcraftPartMissingError(part_name=part_name)
+            raise errors.SnapcraftPartMissingError(part_name=part_name)
         if not full:
             for key in ['description', 'maintainer']:
                 remote_part.pop(key)
@@ -139,16 +139,6 @@ class _RemoteParts(_Base):
         return remote_part
 
 
-class SnapcraftLogicError(Exception):
-
-    @property
-    def message(self):
-        return self._message
-
-    def __init__(self, message):
-        self._message = message
-
-
 class PartsConfig:
 
     def __init__(self, parts, project_options, validator, build_tools,
@@ -188,10 +178,12 @@ class PartsConfig:
             if 'filesets' in properties:
                 del properties['filesets']
 
-            # FIXME: snap is deprecated, rewrite it to prime instead.
-            if properties.get('snap'):
-                deprecations.handle_deprecation_notice('dn1')
-                properties['prime'] = properties.pop('snap')
+            # Handle the deprecated snap keyword.
+            if 'snap' in properties:
+                snap = properties.pop('snap')
+                if snap:
+                    deprecations.handle_deprecation_notice('dn1')
+                    properties['prime'] = snap
 
             self.load_plugin(part_name, plugin_name, properties)
 
@@ -225,7 +217,7 @@ class PartsConfig:
                     top_part = part
                     break
             if not top_part:
-                raise SnapcraftLogicError(
+                raise errors.SnapcraftLogicError(
                     'circular dependency chain found in parts definition')
             sorted_parts = [top_part] + sorted_parts
             self.all_parts.remove(top_part)
@@ -260,7 +252,7 @@ class PartsConfig:
     def validate(self, part_names):
         for part_name in part_names:
             if part_name not in self._part_names:
-                raise EnvironmentError(
+                raise errors.SnapcraftEnvironmentError(
                     'The part named {!r} is not defined in '
                     '{!r}'.format(part_name, self._snapcraft_yaml))
 
@@ -330,7 +322,7 @@ def update():
 def define(part_name):
     try:
         remote_part = _RemoteParts().get_part(part_name, full=True)
-    except SnapcraftPartMissingError as e:
+    except errors.SnapcraftPartMissingError as e:
         raise RuntimeError(
             'Cannot find the part name {!r} in the cache. Please '
             'run `snapcraft update` and try again.\nIf it is indeed missing, '
