@@ -19,7 +19,6 @@ import logging
 import os
 import os.path
 import re
-import sys
 
 import jsonschema
 import yaml
@@ -31,10 +30,10 @@ from snapcraft.internal import (
     errors,
     libraries,
     parts,
-    pluginhandler,
+    states,
 )
 from snapcraft._schema import Validator
-from snapcraft.internal.parts import SnapcraftLogicError, get_remote_parts
+from snapcraft.internal.parts import get_remote_parts
 
 
 logger = logging.getLogger(__name__)
@@ -70,26 +69,6 @@ def _validate_epoch(instance):
     return True
 
 
-class SnapcraftYamlFileError(Exception):
-
-    @property
-    def file(self):
-        return self._file
-
-    def __init__(self, yaml_file):
-        self._file = yaml_file
-
-
-class PluginNotDefinedError(Exception):
-
-    @property
-    def part(self):
-        return self._part
-
-    def __init__(self, part):
-        self._part = part
-
-
 class Config:
 
     @property
@@ -121,13 +100,13 @@ class Config:
 
         snapcraft_yaml = self._process_remote_parts(snapcraft_yaml)
         snapcraft_yaml = self._expand_filesets(snapcraft_yaml)
-        self.data = self._expand_env(snapcraft_yaml)
-
-        self._ensure_no_duplicate_app_aliases()
 
         # both confinement type and build quality are optionals
-        _ensure_confinement_default(self.data, self._validator.schema)
-        _ensure_grade_default(self.data, self._validator.schema)
+        _ensure_confinement_default(snapcraft_yaml, self._validator.schema)
+        _ensure_grade_default(snapcraft_yaml, self._validator.schema)
+
+        self.data = self._expand_env(snapcraft_yaml)
+        self._ensure_no_duplicate_app_aliases()
 
         self.build_tools = self.data.get('build-packages', [])
         self.build_tools.extend(project_options.additional_build_packages)
@@ -141,6 +120,11 @@ class Config:
         if 'architectures' not in self.data:
             self.data['architectures'] = [self._project_options.deb_arch]
 
+    def get_metadata(self):
+        return {'name': self.data['name'],
+                'version': self.data['version'],
+                'arch': self.data['architectures']}
+
     def _ensure_no_duplicate_app_aliases(self):
         # Prevent multiple apps within a snap from having duplicate alias names
         aliases = []
@@ -161,7 +145,7 @@ class Config:
 
         state = {}
         for part in self.parts.all_parts:
-            state[part.name] = part.get_state(step)
+            state[part.name] = states.get_state(part.statedir, step)
 
         return state
 
@@ -183,13 +167,13 @@ class Config:
         return env
 
     def snap_env(self):
-        snap_dir = self._project_options.snap_dir
+        prime_dir = self._project_options.prime_dir
         env = []
 
-        env += _runtime_env(snap_dir, self._project_options.arch_triplet)
+        env += _runtime_env(prime_dir, self._project_options.arch_triplet)
         dependency_paths = set()
         for part in self.parts.all_parts:
-            env += part.env(snap_dir)
+            env += part.env(prime_dir)
             dependency_paths |= part.get_primed_dependency_paths()
 
         # Dependency paths are only valid if they actually exist. Sorting them
@@ -209,6 +193,7 @@ class Config:
             'SNAPCRAFT_STAGE={}'.format(self._project_options.stage_dir),
             'SNAPCRAFT_PROJECT_NAME={}'.format(self.data['name']),
             'SNAPCRAFT_PROJECT_VERSION={}'.format(self.data['version']),
+            'SNAPCRAFT_PROJECT_GRADE={}'.format(self.data['grade']),
         ]
 
     def _expand_env(self, snapcraft_yaml):
@@ -221,6 +206,7 @@ class Config:
                 [
                     ('$SNAPCRAFT_PROJECT_NAME', snapcraft_yaml['name']),
                     ('$SNAPCRAFT_PROJECT_VERSION', snapcraft_yaml['version']),
+                    ('$SNAPCRAFT_PROJECT_GRADE', snapcraft_yaml['grade']),
                     ('$SNAPCRAFT_STAGE', self._project_options.stage_dir),
                 ])
         return snapcraft_yaml
@@ -374,7 +360,8 @@ def get_snapcraft_yaml(base_dir=None):
     snapcraft_yamls = [y for y in possible_yamls if os.path.exists(y)]
 
     if not snapcraft_yamls:
-        raise SnapcraftYamlFileError('snap/snapcraft.yaml')
+        raise errors.SnapcraftYamlFileError(
+            snapcraft_yaml='snap/snapcraft.yaml')
     elif len(snapcraft_yamls) > 1:
         raise errors.SnapcraftEnvironmentError(
             'Found a {!r} and a {!r}, please remove one.'.format(
@@ -396,35 +383,12 @@ def _snapcraft_yaml_load(yaml_file):
         with open(yaml_file, encoding=encoding) as fp:
             return yaml.load(fp)
     except yaml.scanner.ScannerError as e:
-        raise errors.SnapcraftSchemaError(
-            '{} on line {} of {}'.format(
-                e.problem, e.problem_mark.line + 1, yaml_file))
+        raise errors.SnapcraftSchemaError('{} on line {} of {}'.format(
+            e.problem, e.problem_mark.line + 1, yaml_file))
 
 
 def load_config(project_options=None):
-    try:
-        return Config(project_options)
-    except SnapcraftYamlFileError as e:
-        logger.error(
-            "Could not find {}. Are you sure you're in the right directory?\n"
-            "To start a new project, use 'snapcraft init'".format(e.file))
-        sys.exit(1)
-    except errors.SnapcraftSchemaError as e:
-        msg = 'Issues while validating snapcraft.yaml: {}'.format(e.message)
-        logger.error(msg)
-        sys.exit(1)
-    except PluginNotDefinedError as e:
-        logger.error(
-            'Issues while validating snapcraft.yaml: the "plugin" keyword is '
-            'missing for the "{}" part.'.format(e.part))
-        sys.exit(1)
-    except parts.SnapcraftLogicError as e:
-        logger.error('Issue detected while analyzing '
-                     'snapcraft.yaml: {}'.format(e.message))
-        sys.exit(1)
-    except pluginhandler.PluginError as e:
-        logger.error('Issue while loading plugin: {}'.format(e))
-        sys.exit(1)
+    return Config(project_options)
 
 
 def _ensure_confinement_default(yaml_data, schema):
@@ -455,7 +419,7 @@ def _expand_filesets_for(step, properties):
             try:
                 new_step_set.extend(filesets[item[1:]])
             except KeyError:
-                raise SnapcraftLogicError(
+                raise errors.SnapcraftLogicError(
                     '\'{}\' referred to in the \'{}\' fileset but it is not '
                     'in filesets'.format(item, step))
         else:
