@@ -1,6 +1,6 @@
 # -*- Mode:Python; indent-tabs-mode:nil; tab-width:4 -*-
 #
-# Copyright (C) 2015 Canonical Ltd
+# Copyright (C) 2015, 2017 Canonical Ltd
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 3 as
@@ -40,7 +40,6 @@ import glob
 
 import snapcraft
 
-
 logger = logging.getLogger(__name__)
 
 
@@ -52,20 +51,24 @@ class CopyPlugin(snapcraft.BasePlugin):
 
         schema['properties']['files'] = {
             'type': 'object',
+            'additionalProperties': {
+                'type': 'string',
+                'minLength': 1
+            }
         }
 
-        # Inform Snapcraft of the properties associated with building. If these
-        # change in the YAML Snapcraft will consider the build step dirty.
-        schema['build-properties'].append('files')
-
         # The `files` keyword is required here, but the `source` keyword is
         # not. It should default to the current working directory.
         schema['required'].append('files')
-        schema['required'].remove('source')
-        schema['properties']['source']['default'] = '.'
 
         return schema
 
+    @classmethod
+    def get_build_properties(cls):
+        # Inform Snapcraft of the properties associated with building. If these
+        # change in the YAML Snapcraft will consider the build step dirty.
+        return super().get_build_properties() + ['files']
+
     def __init__(self, name, options, project):
         super().__init__(name, options, project)
 
@@ -111,7 +114,9 @@ def _link_or_copy(source, destination, boundary):
         destination_dirname = os.path.dirname(destination)
         normalized = os.path.normpath(os.path.join(destination_dirname, link))
         if os.path.isabs(link) or not normalized.startswith(boundary):
-            follow_symlinks = True
+            # Only follow symlinks that are NOT pointing at libc (LP: #1658774)
+            if link not in snapcraft.repo.Repo.get_package_libraries('libc6'):
+                follow_symlinks = True
 
     snapcraft.common.link_or_copy(source, destination,
                                   follow_symlinks=follow_symlinks)
