@@ -14,11 +14,10 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+import sys as _sys
 
 from snapcraft.internal.sources import Script      # noqa
 from snapcraft.internal.sources import Bazaar      # noqa
-from snapcraft.internal.sources import Deb         # noqa
-from snapcraft.internal.sources import Rpm         # noqa
 from snapcraft.internal.sources import Git         # noqa
 from snapcraft.internal.sources import Mercurial   # noqa
 from snapcraft.internal.sources import Subversion  # noqa
@@ -26,3 +25,6 @@ from snapcraft.internal.sources import Tar         # noqa
 from snapcraft.internal.sources import Local       # noqa
 from snapcraft.internal.sources import Zip         # noqa
 from snapcraft.internal.sources import get         # noqa
+if _sys.platform == 'linux':
+    from snapcraft.internal.sources import Deb     # noqa
+    from snapcraft.internal.sources import Rpm     # noqa
