@@ -37,6 +37,12 @@ class BaseHTTPRequestHandler(http.server.BaseHTTPRequestHandler):
     def log_message(*args):
         logger.debug(args)
 
+    def raise_not_implemented(self, path):
+        logger.error(
+            'Not implemented {} in {} server: {}'.format(
+                path, self.__class__.__name__))
+        raise NotImplementedError(path)
+
 
 class FakeFileHTTPRequestHandler(BaseHTTPRequestHandler):
 
@@ -217,9 +223,7 @@ class FakeSSORequestHandler(BaseHTTPRequestHandler):
         elif parsed_path.path.startswith(tokens_refresh_path):
             self._handle_tokens_refresh_request()
         else:
-            logger.error('Not implemented path in fake SSO server: {}'.format(
-                self.path))
-            raise NotImplementedError(self.path)
+            self.raise_not_implemented(self.path)
 
     def _handle_tokens_discharge_request(self):
         string_data = self.rfile.read(
@@ -298,10 +302,7 @@ class FakeStoreUploadRequestHandler(BaseHTTPRequestHandler):
         if parsed_path.path.startswith('/unscanned-upload/'):
             self._handle_upload_request()
         else:
-            logger.error(
-                'Not implemented path in fake Store Upload server: {}'.format(
-                    self.path))
-            raise NotImplementedError(self.path)
+            self.raise_not_implemented(self.path)
 
     def _handle_upload_request(self):
         logger.info('Handling upload request')
@@ -339,7 +340,7 @@ class FakeStoreAPIRequestHandler(BaseHTTPRequestHandler):
         acl_path = urllib.parse.urljoin(self._DEV_API_PATH, 'acl/')
         account_key_path = urllib.parse.urljoin(
             self._DEV_API_PATH, 'account/account-key')
-        snap_id_path = urllib.parse.urljoin(
+        snaps_path = urllib.parse.urljoin(
             self._DEV_API_PATH, 'snaps/')
         register_path = urllib.parse.urljoin(
             self._DEV_API_PATH, 'register-name/')
@@ -355,11 +356,13 @@ class FakeStoreAPIRequestHandler(BaseHTTPRequestHandler):
             self._handle_acl_request(permission)
         elif parsed_path.path == account_key_path:
             self._handle_account_key_request()
-        elif parsed_path.path.startswith(snap_id_path):
+        elif parsed_path.path.startswith(snaps_path):
             if parsed_path.path.endswith('/builds'):
                 self._handle_sign_build_request()
             elif parsed_path.path.endswith('/close'):
                 self._handle_close_request()
+            else:
+                self.raise_not_implemented(self.path)
         elif parsed_path.path.startswith(register_path):
             self._handle_registration_request()
         elif parsed_path.path.startswith(upload_path):
@@ -369,10 +372,7 @@ class FakeStoreAPIRequestHandler(BaseHTTPRequestHandler):
         elif parsed_path.path.startswith(agreement_path):
             self._handle_sign_request()
         else:
-            logger.error(
-                'Not implemented path in fake Store API server: {}'.format(
-                    self.path))
-            raise NotImplementedError(self.path)
+            self.raise_not_implemented(self.path)
 
     def _handle_refresh(self):
         if self.server.fake_store.needs_refresh:
@@ -578,6 +578,10 @@ class FakeStoreAPIRequestHandler(BaseHTTPRequestHandler):
             self._handle_register_409('already_owned')
         elif data['snap_name'].startswith('test-too-fast'):
             self._handle_register_429('register_window')
+        elif (
+                data['snap_name'].startswith('test_invalid') or
+                len(data['snap_name']) > 40):
+            self._handle_register_name_invalid(data['snap_name'])
         elif data['snap_name'] == 'snap-name-no-clear-error':
             self._handle_unclear_registration_error()
         else:
@@ -603,7 +607,7 @@ class FakeStoreAPIRequestHandler(BaseHTTPRequestHandler):
         self.wfile.write(json.dumps(response).encode())
 
     def _handle_register_429(self, error_code):
-        self.send_response(409)
+        self.send_response(429)
         self.send_header('Content-Type', 'application/json')
         self.end_headers()
         response = {
@@ -614,6 +618,28 @@ class FakeStoreAPIRequestHandler(BaseHTTPRequestHandler):
             response['retry_after'] = 177
         self.wfile.write(json.dumps(response).encode())
 
+    def _handle_register_name_invalid(self, snap_name):
+        self.send_response(400)
+        self.send_header('Content-Type', 'application/json')
+        self.end_headers()
+
+        # Emulates the current Store behaviour and never combines errors.
+        # It's either using invalid chars or too long, never both.
+        if len(snap_name) > 40:
+            msg = ('The name {} should not be longer than 40 characters.'
+                   .format(snap_name))
+        else:
+            msg = (
+                'The name {!r} is not valid. It can only contain dashes, '
+                'numbers and lowercase ascii letters.'.format(snap_name))
+
+        response = {
+            'error_list': [
+                {'code': 'invalid', 'message': msg},
+            ]
+        }
+        self.wfile.write(json.dumps(response).encode())
+
     def _handle_unclear_registration_error(self):
         self.send_response(409)
         self.send_header('Content-Type', 'application/json')
@@ -676,7 +702,9 @@ class FakeStoreAPIRequestHandler(BaseHTTPRequestHandler):
                 'errors': 'Not a valid channel: alpha',
             }
             data = json.dumps(response).encode()
-        elif data['name'] == 'test-snap' or data['name'].startswith('u1test'):
+        elif (
+                data['name'] == 'test-snap' or
+                data['name'].startswith('snapcrafttest')):
             response = {
                 'opened_channels': data['channels'],
                 'channel_map': [
@@ -798,15 +826,7 @@ class FakeStoreAPIRequestHandler(BaseHTTPRequestHandler):
         duplicate_error = urllib.parse.urljoin(
             self._DEV_API_PATH, '/details/upload-id/duplicate-snap')
         account_path = urllib.parse.urljoin(self._DEV_API_PATH, 'account')
-        snap_path = urllib.parse.urljoin(self._DEV_API_PATH, 'snaps')
-        good_validations_path = urllib.parse.urljoin(
-            self._DEV_API_PATH, 'snaps/good/validations')
-        no_validations_path = urllib.parse.urljoin(
-            self._DEV_API_PATH, 'snaps/snap-id/validations')
-        bad_validations_path = urllib.parse.urljoin(
-            self._DEV_API_PATH, 'snaps/bad/validations')
-        err_validations_path = urllib.parse.urljoin(
-            self._DEV_API_PATH, 'snaps/err/validations')
+        snaps_path = urllib.parse.urljoin(self._DEV_API_PATH, 'snaps')
 
         if parsed_path.path.startswith(details_good):
             self._handle_scan_complete_request('ready_to_release', True)
@@ -816,28 +836,25 @@ class FakeStoreAPIRequestHandler(BaseHTTPRequestHandler):
             self._handle_duplicate_snap_request('processing_error', False)
         elif parsed_path.path == account_path:
             self._handle_account_request()
-        elif parsed_path.path.startswith(good_validations_path):
-            self._handle_validation_request('good')
-        elif parsed_path.path.startswith(bad_validations_path):
-            self._handle_validation_request('bad')
-        elif parsed_path.path.startswith(err_validations_path):
-            self._handle_validation_request('err')
-        elif parsed_path.path.startswith(no_validations_path):
-            self._handle_validation_request('no')
-        elif parsed_path.path.startswith(snap_path):
-            if parsed_path.path.endswith('/history'):
+        elif parsed_path.path.startswith(snaps_path):
+            snap_id, request = parsed_path.path.replace(
+                snaps_path + '/', '').split('/')
+            if request == 'history':
                 self._handle_snap_revisions()
-            elif parsed_path.path.endswith('/state'):
+            elif request == 'state':
                 self._handle_snap_status()
+            elif request == 'validations':
+                self._handle_snap_validations(snap_id)
+            elif request == 'developers':
+                self._handle_snap_developers(snap_id)
+            else:
+                self._raise_not_implemented(parsed_path.path)
         else:
-            logger.error(
-                'Not implemented path in fake Store API server: {}'.format(
-                    self.path))
-            raise NotImplementedError(parsed_path)
+            self._raise_not_implemented(parsed_path.path)
 
-    def _handle_validation_request(self, code):
+    def _handle_snap_validations(self, snap_id):
         logger.debug('Handling validation request')
-        if code == 'good':
+        if snap_id == 'good':
             response = [{
                 "approved-snap-id": "snap-id-1",
                 "approved-snap-revision": "3",
@@ -877,13 +894,13 @@ class FakeStoreAPIRequestHandler(BaseHTTPRequestHandler):
             }]
             response = json.dumps(response).encode()
             status = 200
-        elif code == 'bad':
+        elif snap_id == 'bad':
             response = 'foo'.encode()
             status = 200
-        elif code == 'no':
+        elif snap_id == 'test-snap-id-with-no-validations':
             response = json.dumps([]).encode()
             status = 200
-        elif code == 'err':
+        elif snap_id == 'err':
             status = 503
             response = {'error_list': [{'message': 'error'}]}
             response = json.dumps(response).encode()
@@ -892,6 +909,41 @@ class FakeStoreAPIRequestHandler(BaseHTTPRequestHandler):
         self.end_headers()
         self.wfile.write(response)
 
+    def _handle_snap_developers(self, snap_id):
+        logger.debug('Handling snap developers request')
+        if snap_id == 'good':
+            response = json.dumps({'snap_developer': {}}).encode()
+            status = 200
+        elif snap_id == 'test-snap-id-with-dev':
+            payload = {
+                'snap_developer': {
+                    'type': 'snap-developer',
+                    'authority-id': 'dummy',
+                    'publisher-id': 'dummy',
+                    'snap-id': 'test-snap-id-with-dev',
+                    'developers': [{
+                        'developer-id': 'test-dev-id',
+                        'since': '2017-02-10T08:35:00.390258Z',
+                        'until': '2018-02-10T08:35:00.390258Z'
+                    }]
+                }
+            }
+            response = json.dumps(payload).encode()
+            status = 200
+        elif snap_id == 'no-dev':
+            status = 403
+            payload = {'error_list': [
+                {'message': 'error',
+                 'code': 'snap-developer-not-found'}]}
+            response = json.dumps(payload).encode()
+        elif snap_id == 'badrequest':
+            status = 200
+            response = json.dumps({'snap_developer': {}}).encode()
+        self.send_response(status)
+        self.send_header('Content-Type', 'application/json')
+        self.end_headers()
+        self.wfile.write(response)
+
     def _handle_duplicate_snap_request(self, code, can_release):
         logger.debug('Handling duplicate snap request')
         self.send_response(200)
@@ -932,9 +984,24 @@ class FakeStoreAPIRequestHandler(BaseHTTPRequestHandler):
             'basic': {'snap-id': 'snap-id', 'status': 'Approved',
                       'private': False, 'price': None,
                       'since': '2016-12-12T01:01:01Z'},
+            'test-snap-with-no-validations': {
+                'snap-id': 'test-snap-id-with-no-validations',
+                'status': 'Approved',
+                'private': False, 'price': None,
+                'since': '2016-12-12T01:01:01Z'},
+            'test-snap-with-dev': {'snap-id': 'test-snap-id-with-dev',
+                                   'status': 'Approved',
+                                   'private': False, 'price': None,
+                                   'since': '2016-12-12T01:01:01Z'},
             'ubuntu-core': {'snap-id': 'good', 'status': 'Approved',
                             'private': False, 'price': None,
                             'since': '2016-12-12T01:01:01Z'},
+            'core-no-dev': {'snap-id': 'no-dev', 'status': 'Approved',
+                            'private': False, 'price': None,
+                            'since': '2016-12-12T01:01:01Z'},
+            'badrequest': {'snap-id': 'badrequest', 'status': 'Approved',
+                           'private': False, 'price': None,
+                           'since': '2016-12-12T01:01:01Z'},
             }
         snaps.update({
             name: {'snap-id': 'fake-snap-id', 'status': 'Approved',
@@ -1054,24 +1121,18 @@ class FakeStoreAPIRequestHandler(BaseHTTPRequestHandler):
             self._handle_needs_refresh()
             return
         parsed_path = urllib.parse.urlparse(self.path)
-        good_validations_path = urllib.parse.urljoin(
-            self._DEV_API_PATH, 'snaps/good/validations')
-        bad_validations_path = urllib.parse.urljoin(
-            self._DEV_API_PATH, 'snaps/bad/validations')
-        err_validations_path = urllib.parse.urljoin(
-            self._DEV_API_PATH, 'snaps/err/validations')
-
-        if parsed_path.path.startswith(good_validations_path):
-            self._handle_push_validation_request('good')
-        elif parsed_path.path.startswith(bad_validations_path):
-            self._handle_push_validation_request('bad')
-        elif parsed_path.path.startswith(err_validations_path):
-            self._handle_push_validation_request('err')
+        snaps_path = urllib.parse.urljoin(self._DEV_API_PATH, 'snaps')
+        if parsed_path.path.startswith(snaps_path):
+            snap_id, request = parsed_path.path.replace(
+                snaps_path + '/', '').split('/')
+            if request == 'validations':
+                self._handle_push_validation_request(snap_id)
+            elif request == 'developers':
+                self._handle_push_developers_request(snap_id)
+            else:
+                self.raise_not_implemented(self.path)
         else:
-            logger.error(
-                'Not implemented path in fake Store API server: {}'.format(
-                    self.path))
-            raise NotImplementedError(parsed_path)
+            self.raise_not_implemented(self.path)
 
     def _handle_push_validation_request(self, code):
         string_data = self.rfile.read(
@@ -1091,6 +1152,27 @@ class FakeStoreAPIRequestHandler(BaseHTTPRequestHandler):
         self.end_headers()
         self.wfile.write(response)
 
+    def _handle_push_developers_request(self, code):
+        string_data = self.rfile.read(
+            int(self.headers['Content-Length']))
+        if code == 'good':
+            response = string_data
+            status = 200
+        elif code == 'no-dev':
+            response = string_data
+            status = 200
+        elif code == 'badrequest':
+            status = 400
+            response = {'error_list': [
+                {'message': "The given `snap-id` does not match the "
+                            "assertion's.",
+                 'code': 'invalid-request'}]}
+            response = json.dumps(response).encode()
+        self.send_response(status)
+        self.send_header('Content-Type', 'application/json')
+        self.end_headers()
+        self.wfile.write(response)
+
 
 class FakeStoreSearchServer(http.server.HTTPServer):
 
@@ -1117,10 +1199,7 @@ class FakeStoreSearchRequestHandler(BaseHTTPRequestHandler):
             self._handle_download_request(
                 parsed_path.path[len(download_path):])
         else:
-            logger.error(
-                'Not implemented path in fake Store Search server: {}'.format(
-                    self.path))
-            raise NotImplementedError(self.path)
+            self.raise_not_implemented(self.path)
 
     def _handle_details_request(self, package):
         logger.debug(
