@@ -1,6 +1,6 @@
 # -*- Mode:Python; indent-tabs-mode:nil; tab-width:4 -*-
 #
-# Copyright (C) 2016 Canonical Ltd
+# Copyright (C) 2016, 2017 Canonical Ltd
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 3 as
@@ -22,6 +22,7 @@ from unittest import mock
 
 import fixtures
 import pymacaroons
+from testtools.matchers import Contains
 
 from snapcraft import (
     config,
@@ -35,13 +36,17 @@ from snapcraft.storeapi import (
 from snapcraft.tests import fixture_setup
 
 
-class LoginTestCase(tests.TestCase):
+class StoreTestCase(tests.TestCase):
 
     def setUp(self):
         super().setUp()
-        self.useFixture(fixture_setup.FakeStore())
+
+        self.fake_store = self.useFixture(fixture_setup.FakeStore())
         self.client = storeapi.StoreClient()
 
+
+class LoginTestCase(StoreTestCase):
+
     def test_login_successful(self):
         self.client.login(
             'dummy email',
@@ -124,7 +129,7 @@ class LoginTestCase(tests.TestCase):
         self.assertTrue(config.Config().is_empty())
 
 
-class DownloadTestCase(tests.TestCase):
+class DownloadTestCase(StoreTestCase):
 
     # sha512 of snapcraft/tests/data/test-snap.snap
     EXPECTED_SHA512 = (
@@ -132,11 +137,6 @@ class DownloadTestCase(tests.TestCase):
         'd22a956457f14146f7f067b47bd976cf0292f2993ad864ccb498b'
         'fda4128234e4c201f28fe9')
 
-    def setUp(self):
-        super().setUp()
-        self.useFixture(fixture_setup.FakeStore())
-        self.client = storeapi.StoreClient()
-
     def test_download_unexisting_snap_raises_exception(self):
         self.client.login('dummy', 'test correct password')
         e = self.assertRaises(
@@ -241,12 +241,7 @@ class DownloadTestCase(tests.TestCase):
             'test-snap-with-wrong-sha', 'test-channel', download_path)
 
 
-class PushSnapBuildTestCase(tests.TestCase):
-
-    def setUp(self):
-        super().setUp()
-        self.fake_store = self.useFixture(fixture_setup.FakeStore())
-        self.client = storeapi.StoreClient()
+class PushSnapBuildTestCase(StoreTestCase):
 
     def test_push_snap_build_without_login_raises_exception(self):
         self.assertRaises(
@@ -297,12 +292,7 @@ class PushSnapBuildTestCase(tests.TestCase):
         self.client.push_snap_build('snap-id', 'dummy')
 
 
-class GetAccountInformationTestCase(tests.TestCase):
-
-    def setUp(self):
-        super().setUp()
-        self.fake_store = self.useFixture(fixture_setup.FakeStore())
-        self.client = storeapi.StoreClient()
+class GetAccountInformationTestCase(StoreTestCase):
 
     def test_get_account_information_without_login_raises_exception(self):
         self.assertRaises(
@@ -329,7 +319,35 @@ class GetAccountInformationTestCase(tests.TestCase):
                         'private': False,
                         'price': None,
                         'since': '2016-12-12T01:01:01Z',
-                    }
+                    },
+                    'core-no-dev': {
+                        'snap-id': 'no-dev',
+                        'status': 'Approved',
+                        'private': False,
+                        'price': None,
+                        'since': '2016-12-12T01:01:01Z',
+                    },
+                    'badrequest': {
+                        'snap-id': 'badrequest',
+                        'status': 'Approved',
+                        'private': False,
+                        'price': None,
+                        'since': '2016-12-12T01:01:01Z',
+                    },
+                    'test-snap-with-dev': {
+                        'price': None,
+                        'private': False,
+                        'since': '2016-12-12T01:01:01Z',
+                        'snap-id': 'test-snap-id-with-dev',
+                        'status': 'Approved'
+                    },
+                    'test-snap-with-no-validations': {
+                        'price': None,
+                        'private': False,
+                        'since': '2016-12-12T01:01:01Z',
+                        'snap-id': 'test-snap-id-with-no-validations',
+                        'status': 'Approved'
+                    },
                 }
             }
         }, self.client.get_account_information())
@@ -355,19 +373,42 @@ class GetAccountInformationTestCase(tests.TestCase):
                         'private': False,
                         'price': None,
                         'since': '2016-12-12T01:01:01Z',
-                    }
+                    },
+                    'core-no-dev': {
+                        'snap-id': 'no-dev',
+                        'status': 'Approved',
+                        'private': False,
+                        'price': None,
+                        'since': '2016-12-12T01:01:01Z',
+                    },
+                    'badrequest': {
+                        'snap-id': 'badrequest',
+                        'status': 'Approved',
+                        'private': False,
+                        'price': None,
+                        'since': '2016-12-12T01:01:01Z',
+                    },
+                    'test-snap-with-dev': {
+                        'price': None,
+                        'private': False,
+                        'since': '2016-12-12T01:01:01Z',
+                        'snap-id': 'test-snap-id-with-dev',
+                        'status': 'Approved'
+                    },
+                    'test-snap-with-no-validations': {
+                        'price': None,
+                        'private': False,
+                        'since': '2016-12-12T01:01:01Z',
+                        'snap-id': 'test-snap-id-with-no-validations',
+                        'status': 'Approved'
+                    },
                 }
             }
         }, self.client.get_account_information())
         self.assertFalse(self.fake_store.needs_refresh)
 
 
-class RegisterKeyTestCase(tests.TestCase):
-
-    def setUp(self):
-        super().setUp()
-        self.fake_store = self.useFixture(fixture_setup.FakeStore())
-        self.client = storeapi.StoreClient()
+class RegisterKeyTestCase(StoreTestCase):
 
     def test_register_key_without_login_raises_exception(self):
         self.assertRaises(
@@ -413,12 +454,7 @@ class RegisterKeyTestCase(tests.TestCase):
             'The account-key-request assertion is not valid.')
 
 
-class RegisterTestCase(tests.TestCase):
-
-    def setUp(self):
-        super().setUp()
-        self.fake_store = self.useFixture(fixture_setup.FakeStore())
-        self.client = storeapi.StoreClient()
+class RegisterTestCase(StoreTestCase):
 
     def test_register_without_login_raises_exception(self):
         self.assertRaises(
@@ -487,6 +523,28 @@ class RegisterTestCase(tests.TestCase):
             'You must wait 177 seconds before trying to register your '
             'next snap.')
 
+    def test_registering_name_too_long(self):
+        self.client.login('dummy', 'test correct password')
+        name = 'name-too-l{}ng'.format('0' * 40)
+        raised = self.assertRaises(
+            errors.StoreRegistrationError,
+            self.client.register, name)
+        expected = (
+            'The name {} should not be longer than 40 characters.'
+            .format(name))
+        self.assertEqual(str(raised), expected)
+
+    def test_registering_name_invalid(self):
+        self.client.login('dummy', 'test correct password')
+        name = 'test_invalid'
+        raised = self.assertRaises(
+            errors.StoreRegistrationError,
+            self.client.register, name)
+        expected = (
+            'The name {!r} is not valid. It can only contain dashes, numbers '
+            'and lowercase ascii letters.'.format(name))
+        self.assertEqual(str(raised), expected)
+
     def test_unhandled_registration_error_path(self):
         self.client.login('dummy', 'test correct password')
         raised = self.assertRaises(
@@ -495,14 +553,12 @@ class RegisterTestCase(tests.TestCase):
         self.assertEqual(str(raised), 'Registration failed.')
 
 
-class ValidationsTestCase(tests.TestCase):
+class ValidationsTestCase(StoreTestCase):
 
     def setUp(self):
         super().setUp()
         self.fake_logger = fixtures.FakeLogger(level=logging.DEBUG)
         self.useFixture(self.fake_logger)
-        self.fake_store = self.useFixture(fixture_setup.FakeStore())
-        self.client = storeapi.StoreClient()
 
     def test_get_success(self):
         self.client.login('dummy', 'test correct password')
@@ -543,7 +599,7 @@ class ValidationsTestCase(tests.TestCase):
             "revoked": "false",
             "required": True,
         }]
-        result = self.client.get_validations('good')
+        result = self.client.get_assertion('good', 'validations')
         self.assertEqual(result, expected)
 
     def test_get_bad_response(self):
@@ -551,7 +607,7 @@ class ValidationsTestCase(tests.TestCase):
 
         err = self.assertRaises(
             errors.StoreValidationError,
-            self.client.get_validations, 'bad')
+            self.client.get_assertion, 'bad', 'validations')
 
         expected = ("Received error 200: 'Invalid response from the server'")
         self.assertEqual(str(err), expected)
@@ -560,20 +616,19 @@ class ValidationsTestCase(tests.TestCase):
 
     def test_get_error_response(self):
         self.client.login('dummy', 'test correct password')
-        expected = []
 
         err = self.assertRaises(
-            errors.StoreValidationError,
-            self.client.get_validations, 'err')
+            errors.StoreRetryError,
+            self.client.get_assertion, 'err', 'validations')
 
-        expected = ("Received error 503: 'error'")
-        self.assertEqual(str(err), expected)
+        expected = ('too many 503 error responses')
+        self.assertThat(str(err), Contains(expected))
 
     def test_push_success(self):
         self.client.login('dummy', 'test correct password')
         assertion = json.dumps({'foo': 'bar'}).encode('utf-8')
 
-        result = self.client.push_validation('good', assertion)
+        result = self.client.push_assertion('good', assertion, 'validations')
 
         expected = {'assertion': '{"foo": "bar"}'}
         self.assertEqual(result, expected)
@@ -584,7 +639,7 @@ class ValidationsTestCase(tests.TestCase):
 
         err = self.assertRaises(
             errors.StoreValidationError,
-            self.client.push_validation, 'bad', assertion)
+            self.client.push_assertion, 'bad', assertion, 'validations')
 
         expected = ("Received error 200: 'Invalid response from the server'")
         self.assertEqual(str(err), expected)
@@ -597,18 +652,16 @@ class ValidationsTestCase(tests.TestCase):
 
         err = self.assertRaises(
             errors.StoreValidationError,
-            self.client.push_validation, 'err', assertion)
+            self.client.push_assertion, 'err', assertion, 'validations')
 
         expected = ("Received error 501: 'error'")
         self.assertEqual(str(err), expected)
 
 
-class UploadTestCase(tests.TestCase):
+class UploadTestCase(StoreTestCase):
 
     def setUp(self):
         super().setUp()
-        self.fake_store = self.useFixture(fixture_setup.FakeStore())
-        self.client = storeapi.StoreClient()
         self.snap_path = os.path.join(
             os.path.dirname(tests.__file__), 'data',
             'test-snap.snap')
@@ -742,12 +795,7 @@ class UploadTestCase(tests.TestCase):
             self.client.upload, 'test-snap', self.snap_path)
 
 
-class ReleaseTestCase(tests.TestCase):
-
-    def setUp(self):
-        super().setUp()
-        self.fake_store = self.useFixture(fixture_setup.FakeStore())
-        self.client = storeapi.StoreClient()
+class ReleaseTestCase(StoreTestCase):
 
     def test_release_without_login_raises_exception(self):
         self.assertRaises(
@@ -817,14 +865,12 @@ class ReleaseTestCase(tests.TestCase):
             self.client.release, 'test-snap', '10', ['beta'])
 
 
-class CloseChannelsTestCase(tests.TestCase):
+class CloseChannelsTestCase(StoreTestCase):
 
     def setUp(self):
         super().setUp()
         self.fake_logger = fixtures.FakeLogger(level=logging.DEBUG)
         self.useFixture(self.fake_logger)
-        self.fake_store = self.useFixture(fixture_setup.FakeStore())
-        self.client = storeapi.StoreClient()
 
     def test_close_requires_login(self):
         self.assertRaises(
@@ -929,12 +975,10 @@ class MacaroonsTestCase(tests.TestCase):
             storeapi._macaroon_auth, conf)
 
 
-class GetSnapRevisionsTestCase(tests.TestCase):
+class GetSnapRevisionsTestCase(StoreTestCase):
 
     def setUp(self):
         super().setUp()
-        self.fake_store = self.useFixture(fixture_setup.FakeStore())
-        self.client = storeapi.StoreClient()
         self.expected = [{
             'series': ['16'],
             'channels': [],
@@ -1031,12 +1075,10 @@ class GetSnapRevisionsTestCase(tests.TestCase):
             str(e))
 
 
-class GetSnapStatusTestCase(tests.TestCase):
+class GetSnapStatusTestCase(StoreTestCase):
 
     def setUp(self):
         super().setUp()
-        self.fake_store = self.useFixture(fixture_setup.FakeStore())
-        self.client = storeapi.StoreClient()
         self.expected = {
             'channel_map_tree': {
                 'latest': {
@@ -1166,12 +1208,7 @@ class GetSnapStatusTestCase(tests.TestCase):
             str(e))
 
 
-class SignDeveloperAgreementTestCase(tests.TestCase):
-
-    def setUp(self):
-        super().setUp()
-        self.fake_store = self.useFixture(fixture_setup.FakeStore())
-        self.client = storeapi.StoreClient()
+class SignDeveloperAgreementTestCase(StoreTestCase):
 
     def test_sign_dev_agreement_success(self):
         self.client.login('dummy', 'test correct password')
