@@ -250,6 +250,25 @@ class CreateTestCase(CreateBaseTestCase):
         self.assertFalse('icon' in y,
                          'icon found in snap.yaml {}'.format(y))
 
+    def test_version_script(self):
+        self.config_data['version-script'] = 'echo 10.1-devel'
+
+        y = self.generate_meta_yaml()
+
+        self.assertThat(y['version'], Equals('10.1-devel'))
+
+    def test_version_script_exits_bad(self):
+        self.config_data['version-script'] = 'exit 1'
+
+        with testtools.ExpectedException(CommandError):
+            self.generate_meta_yaml()
+
+    def test_version_script_with_no_output(self):
+        self.config_data['version-script'] = 'echo'
+
+        with testtools.ExpectedException(CommandError):
+            self.generate_meta_yaml()
+
     def test_create_meta_with_app(self):
         os.mkdir(self.prime_dir)
         open(os.path.join(self.prime_dir, 'app.sh'), 'w').close()
@@ -404,16 +423,7 @@ class CreateTestCase(CreateBaseTestCase):
 
 class WriteSnapDirectoryTestCase(CreateBaseTestCase):
 
-    scenarios = (
-        ('with build artifacts', dict(build_info='yes')),
-        ('without build artifacts', dict(build_info='')),
-    )
-
     def test_write_snap_directory(self):
-        if self.build_info:
-            self.useFixture(fixtures.EnvironmentVariable(
-                'SNAPCRAFT_BUILD_INFO', self.build_info))
-
         # Setup a snap directory containing a few things.
         _create_file(os.path.join(self.snap_dir, 'snapcraft.yaml'))
         _create_file(
@@ -424,13 +434,6 @@ class WriteSnapDirectoryTestCase(CreateBaseTestCase):
         self.generate_meta_yaml()
         prime_snap_dir = os.path.join(self.prime_dir, 'snap')
 
-        if self.build_info:
-            self.assertThat(os.path.join(prime_snap_dir, 'snapcraft.yaml'),
-                            FileExists())
-        else:
-            self.assertThat(os.path.join(prime_snap_dir, 'snapcraft.yaml'),
-                            Not(FileExists()))
-
         self.assertThat(
             os.path.join(prime_snap_dir, 'hooks', 'test-hook'), FileExists())
         self.assertThat(
@@ -692,7 +695,7 @@ PATH={0}/part1/install/usr/bin:{0}/part1/install/bin
         with open(path, 'rb') as exe:
             self.assertEqual(exe_contents, exe.read())
 
-    @patch('snapcraft.internal.common.run')
+    @patch('snapcraft.internal.common.run_output')
     def test_exe_is_in_path(self, run_mock):
         app_path = os.path.join(self.prime_dir, 'bin', 'app1')
         os.mkdir(os.path.dirname(app_path))
