@@ -21,9 +21,10 @@ import sys
 import tempfile
 import unittest
 import unittest.mock
+from textwrap import dedent
+
 import fixtures
-from testtools import ExpectedException
-from testtools.matchers import Equals
+from testtools.matchers import Contains, Equals, MatchesRegex
 
 import snapcraft
 from snapcraft.internal import dirs, parts
@@ -141,7 +142,7 @@ parts:
             '{alias!r} does not match '.format(
                 path='apps/test/aliases[0]', alias='.test'
             ))
-        self.assertEqual(expected, str(raised)[:len(expected)])
+        self.assertThat(str(raised), Contains(expected))
 
     @unittest.mock.patch('snapcraft.internal.parts.PartsConfig.load_plugin')
     def test_config_loads_plugins(self, mock_loadPlugin):
@@ -160,7 +161,7 @@ parts:
         project_loader.Config()
         mock_loadPlugin.assert_called_with('part1', 'go', {
             'stage-packages': ['fswebcam'],
-            'plugin': 'go', 'stage': [], 'prime': [], 'snap': [],
+            'plugin': 'go', 'stage': [], 'prime': [],
         })
 
     @unittest.mock.patch('snapcraft.internal.parts.PartsConfig.load_plugin')
@@ -183,7 +184,7 @@ parts:
 
         mock_loadPlugin.assert_called_with('part1', 'go', {
             'source': 'http://source.tar.gz', 'stage-packages': ['fswebcam'],
-            'plugin': 'go', 'stage': [], 'prime': [], 'snap': []})
+            'plugin': 'go', 'stage': [], 'prime': []})
 
     @unittest.mock.patch('snapcraft.internal.parts.PartsConfig.load_plugin')
     def test_config_composes_with_remote_subpart(self, mock_loadPlugin):
@@ -205,7 +206,7 @@ parts:
 
         mock_loadPlugin.assert_called_with('part1', 'go', {
             'source': 'http://source.tar.gz', 'stage-packages': ['fswebcam'],
-            'plugin': 'go', 'stage': [], 'prime': [], 'snap': []})
+            'plugin': 'go', 'stage': [], 'prime': []})
 
     @unittest.mock.patch('snapcraft.internal.parts.PartsConfig.load_plugin')
     def test_chaining_remotes_not_locally_declared(self, mock_loadPlugin):
@@ -247,13 +248,12 @@ parts:
         parts.update()
 
         raised = self.assertRaises(
-            parts.SnapcraftPartMissingError,
+            errors.SnapcraftPartMissingError,
             project_loader.Config)
-        self.assertEqual(
+        self.assertThat(str(raised), Contains(
             'Cannot find the definition for part {!r}.\n'
             'It may be a remote part, run `snapcraft update` to refresh '
-            'the remote parts cache.'.format('non-existing-part'),
-            str(raised))
+            'the remote parts cache.'.format('non-existing-part')))
 
     def test_config_after_is_an_undefined_part(self):
         self.useFixture(fixture_setup.FakeParts())
@@ -273,13 +273,12 @@ parts:
         parts.update()
 
         raised = self.assertRaises(
-            parts.SnapcraftPartMissingError,
+            errors.SnapcraftPartMissingError,
             project_loader.Config)
-        self.assertEqual(
+        self.assertThat(str(raised), Contains(
             'Cannot find the definition for part {!r}.\n'
             'It may be a remote part, run `snapcraft update` to refresh '
-            'the remote parts cache.'.format('non-existing-part'),
-            str(raised))
+            'the remote parts cache.'.format('non-existing-part')))
 
     @unittest.mock.patch('snapcraft.internal.pluginhandler.load_plugin')
     def test_config_uses_remote_part_from_after(self, mock_load):
@@ -318,7 +317,7 @@ parts:
             'curl',
             plugin_name='autotools',
             part_properties={
-                'plugin': 'autotools', 'stage': [], 'prime': [], 'snap': [],
+                'plugin': 'autotools', 'stage': [], 'prime': [],
                 'source': 'http://curl.org'},
             project_options=project_options,
             part_schema=self.part_schema,
@@ -327,7 +326,7 @@ parts:
             'part1',
             plugin_name='go',
             part_properties={
-                'plugin': 'go', 'stage': [], 'prime': [], 'snap': [],
+                'plugin': 'go', 'stage': [], 'prime': [],
                 'stage-packages': ['fswebcam']},
             project_options=project_options,
             part_schema=self.part_schema,
@@ -357,7 +356,8 @@ parts:
         config = project_loader.Config(project_options)
 
         self.assertEqual(config.parts.build_tools,
-                         ['gcc-arm-linux-gnueabihf'])
+                         ['gcc-arm-linux-gnueabihf',
+                          'libc6-dev-armhf-cross'])
 
     def test_config_has_no_extra_build_tools_when_not_cross_compiling(self):
         class ProjectOptionsFake(snapcraft.ProjectOptions):
@@ -403,7 +403,7 @@ parts:
     after: [p1]
 """)
         raised = self.assertRaises(
-            parts.SnapcraftLogicError,
+            errors.SnapcraftLogicError,
             project_loader.Config)
 
         self.assertEqual(
@@ -434,6 +434,44 @@ parts:
         self.assertEqual(raised.message,
                          "'name' is a required property")
 
+    def test_get_metadata(self):
+        self.make_snapcraft_yaml("""name: test
+version: "1"
+summary: test
+description: nothing
+architectures: ['amd64']
+confinement: strict
+grade: stable
+
+parts:
+  part1:
+    plugin: go
+    stage-packages: [fswebcam]
+""")
+        config = project_loader.load_config()
+        metadata = config.get_metadata()
+        self.assertEqual(metadata['name'], 'test')
+        self.assertEqual(metadata['version'], '1')
+        self.assertEqual(metadata['arch'], ['amd64'])
+
+    def test_version_script(self):
+        self.make_snapcraft_yaml("""name: test
+version: "1.0"
+version-script: echo $SNAPCRAFT_PROJECT_VERSION-$SNAPCRAFT_PROJECT_GRADE
+summary: test
+description: nothing
+architectures: ['amd64']
+confinement: strict
+grade: devel
+
+parts:
+  part1:
+    plugin: go
+    stage-packages: [fswebcam]
+""")
+        config = project_loader.load_config()
+        self.assertEqual(config.data['version-script'], 'echo 1.0-devel')
+
     @unittest.mock.patch('snapcraft.internal.parts.PartsConfig.load_plugin')
     def test_invalid_yaml_invalid_name_as_number(self, mock_loadPlugin):
         fake_logger = fixtures.FakeLogger(level=logging.ERROR)
@@ -657,7 +695,7 @@ parts:
         raised = self.assertRaises(
             errors.SnapcraftSchemaError,
             project_loader.Config)
-        self.assertThat(str(raised), Equals(
+        self.assertThat(str(raised), Contains(
             "The 'parts/part1/organize/foo' property does not match the "
             "required schema: None is not of type 'string'"))
 
@@ -681,7 +719,7 @@ parts:
         raised = self.assertRaises(
             errors.SnapcraftSchemaError,
             project_loader.Config)
-        self.assertThat(str(raised), Equals(
+        self.assertThat(str(raised), Contains(
             "The 'parts/part1/organize/foo' property does not match the "
             "required schema: '' is too short (minimum length is 1)"))
 
@@ -714,7 +752,6 @@ parts:
         project_loader.Config()
 
         mock_loadPlugin.assert_called_with('part1', 'go', {
-            'snap': [],
             'prime': ['/usr/lib/wget.so', '/usr/bin/wget',
                       '/usr/share/my-icon.png'],
             'plugin': 'go', 'stage-packages': ['fswebcam'],
@@ -783,7 +820,7 @@ parts:
 
         mock_load_plugin.assert_called_with('main', 'make', {
             'source': 'project-name-1',
-            'plugin': 'make', 'stage': [], 'prime': [], 'snap': [],
+            'plugin': 'make', 'stage': [], 'prime': [],
             'make-options': ['DEP={}'.format(self.stage_dir)],
         })
 
@@ -877,7 +914,7 @@ parts:
 
         mock_loadPlugin.assert_called_with('part1', 'go', {
             'plugin': 'go', 'stage-packages': ['fswebcam'],
-            'stage': [], 'prime': [], 'snap': [],
+            'stage': [], 'prime': [],
         })
 
 
@@ -1270,10 +1307,11 @@ class InitTestCase(tests.TestCase):
         """Test that an error is raised if snap/snapcraft.yaml is missing"""
 
         raised = self.assertRaises(
-            project_loader.SnapcraftYamlFileError,
+            errors.SnapcraftYamlFileError,
             project_loader.Config)
 
-        self.assertEqual(raised.file, os.path.join('snap', 'snapcraft.yaml'))
+        self.assertThat(str(raised), Contains(
+            os.path.join('snap', 'snapcraft.yaml')))
 
     def test_both_new_and_old_yamls_cause_error(self):
         os.mkdir('snap')
@@ -1885,21 +1923,22 @@ class ValidationTestCase(ValidationBaseTestCase):
             errors.SnapcraftSchemaError,
             project_loader.Validator(self.data).validate)
 
-        self.assertEqual(
+        self.assertThat(str(raised), Contains(
             "The 'apps/service1/restart-condition' property does not match "
             "the required schema: 'on-watchdog' is not one of ['on-success', "
-            "'on-failure', 'on-abnormal', 'on-abort', 'always', 'never']",
-            str(raised))
+            "'on-failure', 'on-abnormal', 'on-abort', 'always', 'never']"))
 
     def test_both_snap_and_prime_specified(self):
         self.data['parts']['part1']['snap'] = ['foo']
         self.data['parts']['part1']['prime'] = ['bar']
 
-        with ExpectedException(
-                errors.SnapcraftSchemaError,
-                "The 'parts/part1' property does not match the required "
-                "schema: .* cannot contain both 'snap' and 'prime' keywords."):
-            project_loader.Validator(self.data).validate()
+        raised = self.assertRaises(
+            errors.SnapcraftSchemaError,
+            project_loader.Validator(self.data).validate)
+
+        self.assertThat(str(raised), MatchesRegex(
+            ".*The 'parts/part1' property does not match the required "
+            "schema: .* cannot contain both 'snap' and 'prime' keywords.*"))
 
 
 class DaemonDependencyTestCase(ValidationBaseTestCase):
@@ -1926,10 +1965,9 @@ class DaemonDependencyTestCase(ValidationBaseTestCase):
             errors.SnapcraftSchemaError,
             project_loader.Validator(self.data).validate)
 
-        self.assertEqual(
+        self.assertThat(str(raised), Contains(
             "The 'apps/service1' property does not match the required schema: "
-            "'daemon' is a dependency of '{}'".format(self.option),
-            str(raised))
+            "'daemon' is a dependency of '{}'".format(self.option)))
 
 
 class RequiredPropertiesTestCase(ValidationBaseTestCase):
@@ -2042,43 +2080,32 @@ class InvalidAppNamesTestCase(ValidationBaseTestCase):
 
 class TestPluginLoadingProperties(tests.TestCase):
 
-    def setUp(self):
-        super().setUp()
-        dirs.setup_dirs()
+    scenarios = [
+        ('slots', dict(property='slots')),
+        ('plugs', dict(property='plugs')),
+    ]
 
-        self.data = """name: my-package-1
-version: 1.0-snapcraft1~ppa1
-summary: my summary less that 79 chars
-description: description which can be pretty long
-parts:
-    part1:
-        plugin: nil
-"""
+    def test_loading_properties(self):
+        self.make_snapcraft_yaml(dedent("""\
+            name: my-package-1
+            version: 1.0-snapcraft1~ppa1
+            summary: my summary less that 79 chars
+            description: description which can be pretty long
+            parts:
+                part1:
+                    plugin: nil
+                    {property}: [{property}1]
+            """).format(property=self.property))
 
-        self.fake_logger = fixtures.FakeLogger(level=logging.ERROR)
-        self.useFixture(self.fake_logger)
-        self.expected_message_template = (
-            "Issue while loading plugin: properties failed to load for "
+        expected_message = (
+            "Issue while loading part: properties failed to load for "
             "part1: Additional properties are not allowed ('{}' was "
-            "unexpected)\n")
+            "unexpected)").format(self.property)
 
-    def test_slots_as_properties_should_fail(self):
-        self.data += '        slots: [slot1]'
-        self.make_snapcraft_yaml(self.data)
+        raised = self.assertRaises(errors.PluginError,
+                                   project_loader.load_config)
 
-        self.assertRaises(SystemExit, project_loader.load_config)
-
-        expected_message = self.expected_message_template.format('slots')
-        self.assertIn(expected_message, self.fake_logger.output)
-
-    def test_plugs_as_properties_should_fail(self):
-        self.data += '        plugs: [plug1]'
-        self.make_snapcraft_yaml(self.data)
-
-        self.assertRaises(SystemExit, project_loader.load_config)
-
-        expected_message = self.expected_message_template.format('plugs')
-        self.assertIn(expected_message, self.fake_logger.output)
+        self.assertThat(str(raised), Contains(expected_message))
 
 
 class TestFilesets(tests.TestCase):
@@ -2109,14 +2136,13 @@ class TestFilesets(tests.TestCase):
         self.properties['stage'] = ['$3']
 
         raised = self.assertRaises(
-            parts.SnapcraftLogicError,
+            errors.SnapcraftLogicError,
             project_loader._expand_filesets_for,
             'stage', self.properties)
 
-        self.assertEqual(
-            raised.message,
+        self.assertThat(str(raised), Contains(
             '\'$3\' referred to in the \'stage\' fileset but it is not '
-            'in filesets')
+            'in filesets'))
 
 
 class SnapcraftEnvTestCase(tests.TestCase):
